// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// Module: BinnedProxyResponse
// Author: Hassan Siddiqui
//
// Description:
// Provides a proxy layer to any GenericResponse object.
// This later rebins the raw data to suit the rebin
// factor specified.
//
// Comments:
//
// Modifications: See ChangeLog
//
// $Id: BinnedProxyResponse.h,v 1.18 2009/09/03 14:18:07 aibarra Exp $

#ifndef BinnedProxyResponse_H
#define BinnedProxyResponse_H

/* General */

/* SAS-specific */
#include "Tracer.h"

/* Local */
#include "GenericResponse.h"

/********************************
 *
 * class BinnedProxyResponse
 *
 * Provides a proxy layer to any GenericResponse object.
 * This later rebins the raw data to suit the rebin
 * factor specified.
 *
 *********************************/

class BinnedProxyResponse : public GenericResponse
{

public:

  /* Constructors */
  BinnedProxyResponse (GenericResponse * response,
		       unsigned binfactor,
		       unsigned offset,
		       unsigned firstchannel,
		       unsigned detchans);

  /* Destructors */
  ~BinnedProxyResponse();


  /** Channel range accessors */
  /* AI gcc4 porting
   * error: type qualifiers ignored on function return type
   * => leading const removed
   */
   unsigned int   firstChannel() const;
   unsigned int   lastChannel() const;


  /** Response Accessors */
  unsigned int                 ebinFirstIndex() const;
  unsigned int                 numberOfEbins() const;
  const Bin ebins(unsigned int row) const;
  const vector<double>         responseVector ( unsigned int row ) const;
  const vector<double>         responseVector ( double energy ) const;

  /* Ebounds Accessor */
  const BinnedAxis * ebounds() const;

  unsigned int energyToPha( double energy ) const;

private:

  GenericResponse * _response;
  unsigned _binFactor;
  unsigned _offset;

  unsigned _firstChannel;
  unsigned _detchans;
  unsigned _lastChannel;

  BinnedAxis * _ebounds;
};

#endif




