// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// 
// Module: CalMOSResponse
// Author: Hassan Siddiqui
//
// Description: 
// Provides a clean interface to CAL response data.
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: CalMOSResponse.cc,v 1.17 2007/06/06 12:42:14 eojero Exp $

/* General */
#include <string>
#include <iostream>
#include <vector>

/* SAS-specific */
#include "Dal.h"
#include "CalServer.h"
#include "Redistribution.h"
#include "Tracer.h"
#include "DetectorMap.h"

// Local classes
#include "CalMOSResponse.h"

/*****************************
 *
 * class CalMOSResponse
 *
 * Interface to CAL response data.
 *
 *****************************/

/* Destructor */
CalMOSResponse::~CalMOSResponse()
{
  PROC;
  ENDPROC;
}

inline void CalMOSResponse::init()
{
  
  PROC;

  /* Currently set state for default EMOS settings */
  calServer.state()->node()->set(EMOScam::PRIMARY_NODE);
  
  /* Get response information */
  CanonicalRedist * rmf = 0; rmf = calServer.getAtom(rmf);
  _rmf = rmf;
  _rmf->at(300,300);

  
  ENDPROC;
}

/** Constructors */

/* Provides the entire range of 
 * redistribution available from the CAL */
CalMOSResponse::CalMOSResponse ( unsigned       lowerPI,
				 unsigned       nOutputChannels,
				 unsigned       outputChannelWidth,
                                EventGrades    eventGrade)
{
  init();

  _eventGrade=eventGrade;

  createWithCalEbins(lowerPI, 
		     nOutputChannels, 
		     outputChannelWidth);
}  

/* Provides a restricted range */
CalMOSResponse::CalMOSResponse ( unsigned       lowerPI,
				 unsigned       nOutputChannels,
				 unsigned       outputChannelWidth,
				 const BinnedAxis * ebins,
                                 EventGrades    eventGrade)
{
  init();

  _eventGrade=eventGrade;

  createWithUserEbins( lowerPI, 
		       nOutputChannels, 
		       outputChannelWidth,
		       ebins);
}

/* 
   Method: spatialAverage
   Purpose: Uses a detector map to calculate the spatially averaged
            redistribution parameters 
*/
void CalMOSResponse::spatialAverage(DetectorMap *detmap) const
{
  PROC;

   /* Use the detector map to spatially average the RMF */
    _rmf->spatialAverage(detmap->rawX(), detmap->rawY(), detmap->weight());

  ENDPROC
}

