// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: CalPNResponse
// Author: Hassan Siddiqui
//
// Description: 
// Provides a clean interface to CAL response data.
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: CalPNResponse.h,v 1.17 2007/06/06 12:42:14 eojero Exp $

#ifndef CalPNResponse_H
#define CalPNResponse_H

// General classes

#include <string>
#include <iostream>
#include <vector>

// SAS-specific classes
#include "CalServer.h"
#include "Redistribution.h"

// Utility functions
#include "Tracer.h"

// Local classes
#include "CalResponse.h"
#include "PixCoords.h"

/********************************
 *
 * class CalPNResponse
 *
 * Interface to Cal response data.
 *
 *********************************/

class CalPNResponse : public CalResponse
{

public:

  /** Constructors */

  /* Provides the entire range of redistribution available from the CAL */
 
  CalPNResponse (  PixCoords cenRaw,
		   unsigned minOutputChannel,
		   unsigned nOutputChannels,
		   unsigned outputChannelWidth,
		   EventGrades eventGrade);

  /* Provides a restricted range */
  CalPNResponse (  PixCoords cenRaw,
		   unsigned minOutputChannel,
		   unsigned nOutputChannels,
		   unsigned outputChannelWidth,
		   const BinnedAxis * ebins,
		   EventGrades eventGrade);

  /* Perform a spatial averaging of the redistribution params */
  void spatialAverage(DetectorMap *_detmap) const;

  
  /** Destructors */
  ~CalPNResponse();

private:

  /** Convenience Method for initialisation */
  void init( PixCoords cenRaw );

};

#endif




