// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by
//            pkgmaker 0.1
//            Thursday, December 11 1997
//
// Module: CalResponse
// Author: Hassan Siddiqui
//
// Description:
// Provides a clean interface to CAL response data.
//
// Comments:
//
// Modifications: See ChangeLog
//
// $Id: CalResponse.h,v 1.18 2009/09/03 14:18:07 aibarra Exp $

#ifndef CalResponse_H
#define CalResponse_H

// General classes

#include <string>
#include <iostream>
#include <vector>

// SAS-specific classes
#include "CalServer.h"
#include "Redistribution.h"
#include "DetectorMap.h"

// Utility functions
#include "Tracer.h"

// Local classes
#include "GenericResponse.h"

/********************************
 *
 * class CalResponse
 *
 * Interface to Cal response data.
 *
 *********************************/

class CalResponse : public GenericResponse
{

public:

  void check();

  /** Constructors */
  CalResponse();

  /** Initialisation routines - called from derived classes. */

  /* Provides the entire range of redistribution available from the CAL */

  /* Initialise using CAL ebins */
   void createWithCalEbins (
		unsigned minOutputChannel,
		unsigned nOutputChannels,
		unsigned outputChannelWidth);

/* Initialise using user-defined ebins */
  void createWithUserEbins (
			    unsigned minOutputChannel,
			    unsigned nOutputChannels,
			    unsigned outputChannelWidth,
			    const BinnedAxis * ebins
			    );

  /* Destructors */
  ~CalResponse();


  /** Channel range accessors */
  /* AI gcc4 porting
   * error: type qualifiers ignored on function return type
   * => leading const removed
   */
   unsigned   firstChannel() const;
   unsigned   lastChannel() const;


  /** Response Accessors */
  unsigned                 ebinFirstIndex() const;
  unsigned                 numberOfEbins() const;
  const Bin ebins(unsigned row) const;
  const vector<double>         responseVector ( unsigned row ) const;
  const vector<double>         responseVector ( double energy ) const;

  /* Ebounds Accessor
   * Note the size and range corresponds to that specified for the channel,
   * so no accessors are needed to cover this. */

  const BinnedAxis * ebounds() const;

  /** Modifiers */
  /* Perform a spatial averaging of the response parameters.
     This method superceded by MOS and PN response classes */
  void spatialAverage(DetectorMap* /* *_detmap */) const;

private:

  /* Core initialisation */
   void create (
		unsigned minOutputChannel,
		unsigned nOutputChannels,
		unsigned outputChannelWidth);


  /* Pad vector with zeros, or truncate vector, to a given size. */
  const vector<double> resizeVector( const vector<double> & in,
				     unsigned nelements) const;


  unsigned _lastChannel;

  double _energyLow;
  double _energyHigh;

  unsigned _rowLow;
  unsigned _rowHigh;
  unsigned _nrows;

protected:

  CanonicalRedist *  _rmf;
  const BinnedAxis * _ebins;
  const BinnedAxis * _eboundsAxis;
  EventGrades _eventGrade;
};

#endif




