// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// 
// Module: ConcreteRmfDataServer
// Author: Hassan Siddiqui
//
// Description: Data pool which is used during arf generation
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: ConcreteRmfDataServer.h,v 1.18 2013/02/15 15:04:00 rsaxton Exp $

#ifndef ConcreteRmfDataServer_H
#define ConcreteRmfDataServer_H

/* General classes */

/* SAS-specific classes */
#include "Tracer.h"

/* Local classes */
#include "RmfDataServer.h"
#include "GenericResponse.h"
#include "ResponseCorrector.h"
#include "CorePosData.h"
#include "PhotonDistribution.h"
#include "PixCoords.h"

/*******************************
 *
 * class ConcreteRmfDataServer
 *
 * Data pool which is used during rmf generation
 *
 ********************************/

class ConcreteRmfDataServer : public RmfDataServer
{
public:

  /* Constructors */ 
  ConcreteRmfDataServer( const string& instrument,
			 unsigned      tlmin,
			 unsigned      tlmax,
			 unsigned      detchans,
			 unsigned      binfactor,
			 unsigned      offset,
			 double        energyLow,
			 double        energyHigh,
			 unsigned      nEnergyBins,
			 bool          withenergyranges,
                         CorePosData   *corePosData,
                         const string& submode,
                         const string& format,
                         bool          correctForPileUp,
                         const string& eventfile,
                         PixelIterator *pxitr );

  /* Destructors */
   ~ConcreteRmfDataServer();

  /* Accessors */
  MatrixColumn             *  matrixColumn( Table * table,
					    const vector<unsigned long>& sizeVect ) const;
  Xmm::Instrument            instrumentID()      const;   
  const string               instrument()        const; 
  const PhotonDistribution * photonDistibution() const; 
  const GenericResponse    * response()          const; 
  const ResponseCorrector  * responseCorrector() const; 
  const Frame              * frame()             const;  

  /* Spectrum-related properties */
  unsigned                   tlmin()             const;
  unsigned                   tlmax()             const;

private:

  GenericResponse    * _response; /* Rebinned response */
  //  GenericResponse    * _response_in; /* Actual response */
  PhotonDistribution * _photonDistribution;
  Frame              * _frame;
  ResponseCorrector  * _responseCorrector;

  string              _instrument;
  Xmm::Instrument     _instrumentID;

  unsigned           _tlmin;
  unsigned           _tlmax;

  bool               _fixedlength; // Fixed or Variable length vector column

  BinnedAxis        * _ebins;
  bool                _withebins;

};

#endif
