// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// Module: DetectorMap
// Author: Richard Saxton
//
// Description:
//
// A detector map
//
// Modifications: See ChangeLog
//
#ifndef DetectorMap_H
#define DetectorMap_H

// General classes
#include <iostream>

// SAS-specific classes

// General utility functions

// Local classes
#include "DatafileAttributes.h"
#include "PixelIterator.h"
#include "DataSubspace.h"
#include "RegionExtent.h"

class DetectorMap
{
public:

  /* Constructor */
  DetectorMap(PixelIterator * const filteredDetmap, const PositionUtils *putils);

  /* Destructor */
  ~DetectorMap();

  /* Modifiers */
   /* Calculate RAW coords from the filtered detmap */
  void setRaw(const PositionUtils *putils);

  /* Accessors to the filtered detmap and components */
  /* AI gcc4 porting
   * error: type qualifiers ignored on function return type
   * => trailing const removed
   */
  PixelIterator *  filteredMap();

  vector<double> detX() const;
  vector<double> detY() const;
  vector<double> weight() const;
  vector<unsigned short> rawX() const;
  vector<unsigned short> rawY() const;
  PixCoords wtmean_centre_raw() const;

private:

  PixelIterator * _filteredDetmap;
  bool _rawSet;
  vector<double> _detx,
                 _dety,
                 _weight;

  vector<unsigned short> _rawx,
                         _rawy;

};

#endif
