// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by
//            pkgmaker 0.1
//            Thursday, December 11 1997
//
// Module: DummyResponse
// Author: Hassan Siddiqui
//
// Description:
// Provides a clean interface to Dummy response data.
//
// Comments:
//
// Modifications: See ChangeLog
//
// $Id: DummyResponse.h,v 1.18 2009/09/03 14:18:07 aibarra Exp $

#ifndef DummyResponse_H
#define DummyResponse_H

/* General classes */

/* SAS-specific classes */
#include "Dal.h"

/* Utility functions */

/* Local classes */
#include "GenericResponse.h"

/*****************************
 *
 * class DummyResponse
 *
 * Interface to Dummy response data.
 *
 *****************************/


class DummyResponse : public GenericResponse
{

public:

  /* Constructor */

  DummyResponse( unsigned int nchannels,
		 double energyLowKeV,
		 double energyHighKeV,
		 unsigned int numberOfEbins
		 );

  ~DummyResponse();

  /** Channel range accessors */
  /* AI gcc4 porting
   * error: type qualifiers ignored on function return type
   * => leading const removed
   */
   unsigned int   firstChannel()     const;
   unsigned int   lastChannel()      const;


  /** Response Accessors */
  unsigned int numberOfEbins()    const;

  const Bin ebins (unsigned int row) const;

  const vector<double>  responseVector ( unsigned int energy_iterator ) const;

  const vector<double>  responseVector ( double energy ) const;

  /** Ebounds Accessors */
  const BinnedAxis * ebounds()  const;

  unsigned int    energyToPha( double energy ) const;

private:

  double _energyLow;
  double _energyHigh;
  unsigned int _numberOfEbins;

  unsigned int _firstChannel;
  unsigned int _lastChannel;

  BinnedAxis *_ebins;
  BinnedAxis *_ebounds;

  /** Constants */
#if __GNUC__ >= 6 
  static constexpr double GAUSSIAN_SIGMA = 10;
  static constexpr double _ENERGYRES_CONST = 1.7;
#else
  static const double GAUSSIAN_SIGMA = 10;
  static const double _ENERGYRES_CONST = 1.7;
#endif
};

#endif




