// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// Module: FixedMatrixColumn
// Author: Hassan Siddiqui
//
// Description:
// Fixed length vector column representation of MatrixColumn.
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: FixedMatrixColumn.h,v 1.17 2007/06/06 12:42:14 eojero Exp $

#ifndef FixedMatrixColumn_H
#define FixedMatrixColumn_H

/* General */

/* SAS-specific */
#include "Dal.h"

/* Local */
#include "MatrixColumn.h"

/*******************************
 *
 * class FixedMatrixColumn
 *
 *******************************/

class FixedMatrixColumn : public MatrixColumn
{

public:

  /* Constructor */
  FixedMatrixColumn( Table * table,
		const vector<unsigned long>& sizeVect);

  /* Destructor */
  ~FixedMatrixColumn();

  /* Set state to a particular row, and the size of that row */
   void row( unsigned long row,
	     unsigned int elements ); 

  /* Write to a given position in row */
   void writeTo( unsigned int element,
		 double value) ;

  /* The number of elements in row */
   unsigned int rowElements() const ;

  /* Inform class that processing for current row is complete */
   void closeRow();

private:

  Column     * _matrixCol;

  ColumnData * _matrixColData;

  real32     * _matrixData;
  
  unsigned long _row;
  unsigned int _rowelements;

  /* Keep trace of element furthest along row that has been written to. When row is closed, the row will be padded with zeros, from the next element on, to _rowelements. */
  unsigned int _largestelement;
};

#endif
