// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// Module: MatrixColumn
// Author: Hassan Siddiqui
//
// Description:
// Special class to handle creation, reading and writing to the RMF
// MATRIX column. This is special as it can be of variable or fixed
// length vector type.
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: MatrixColumn.h,v 1.17 2007/06/06 12:42:14 eojero Exp $

#ifndef MatrixColumn_H
#define MatrixColumn_H

/* General */

/* SAS-specific */
#include "Dal.h"

/* Local */

/*******************************
 *
 * class MatrixColumn
 *
 *******************************/

class MatrixColumn
{

public:

  /* Constructor */
  MatrixColumn() {PROC; ENDPROC;}

  /* Destructor */
  virtual ~MatrixColumn(){PROC; ENDPROC;}

  /* Set state to a particular row, and the size of that row */
  virtual void row( unsigned long row,
		    unsigned int elements ) = 0;

  /* Write to a given position in row */
  virtual void writeTo( unsigned int element,
			double value) = 0;

  /* The number of elements in row */
  virtual unsigned int rowElements() const = 0;

  /* Inform class that processing for current row is complete */
  virtual void closeRow() = 0;

protected:

private:

};

#endif
