// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: PattFracs
// Author: Richard Saxton
//
// Description: 
// Provides an clean interface to a fluxed spectrum or to an ARF
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: PattFracs.h,v 1.1 2013/02/15 15:05:54 rsaxton Exp $

#ifndef PattFracs_H
#define PattFracs_H

// General classes
#include<cassert>

#include <string>
#include <iostream>
#include <vector>
#include <map>

// SAS-specific classes

#include "Dal.h"
#include "errstr.h"
#include "EnergySpectrum.h"
#include "CalServer.h"
#include "PixCoords.h"

// Utility functions
#include "Tracer.h"

// -----------------------------------------------------------------------------------

// class PattFracs

// Supports data consisting of an energy, energy width and data value

// -----------------------------------------------------------------------------------

class PattFracs 
{
public:

  /* Constructor - from a fluxed spectrum */
  //PattFracs(const unsigned short chipX, const unsigned short chipY, EnergySpectrum input_spectrum);
  PattFracs(PixCoords chipCoords);

  /* Destructor */
  ~PattFracs();

  /* Member fields */
  /* Methods */


// Method: fraction - return the pattern fraction for a given pattern and engy
  double fraction(unsigned p, double energy) const;

// Method: weightedFractions - returns the average fraction weighted by an
//                             input spectrum
  vector<double> weightedFractions(EnergySpectrum spectrum) const;

// Method: value - inherited from EnergySpectrum returns the 
//                 count rate / cm^2 in each bin

private:

  // Suppress default constructor
  PattFracs();

  // Variables
  vector<CcdQuantumEfficiencyCurve> _pcurve;
  unsigned _N;  // Number of patterns

};


#endif




