// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// Module: ResponseRow.h
// Author: Hassan Siddiqui
//
// Description: Classes which allow the response matrix to be groups.
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: ResponseRow.h,v 1.17 2007/06/06 12:42:14 eojero Exp $

#ifndef ResponseRow_H
#define ResponseRow_H


/* General classes */
#include "list"

/* SAS-specific classes */
#include "Dal.h"
#include "Tracer.h"

/* Local classes */
#include "ResponseGrouping.h"
#include "MatrixColumn.h"

/***********************************************************
 *
 * class ResponseRow
 *
 * Contains statistics (# groups, # elements > threshold ) 
 * and grouped data for a row within the response matrix 
 * Later, this might be split into two classes: none which generates the statistics
 * and the other which generates statistics as well as the grouped data structure.
 *
 ***********************************************************/

class ResponseRow : public list<ResponseGroup>
{

 public:

  /* Use constructor using as input the row of interest 
   * of the raw rmf as input to generate a grouped response.
   */

  ResponseRow( const vector<double>& response, double threshold); 

  const vector<double> expanded() const;

  void writeTo(unsigned long row,
	       MatrixColumn * matrixColData, 
	       ColumnData * n_grpColData,
	       ColumnData * f_chanColData,
	       ColumnData * n_chanColData,
               unsigned     tlmin ) const;

  /* Accessors */

  int ngroups() const;

  int nresponse_elements() const;

  double threshold() const;

  /* Suppress default constructor */
  ResponseRow();

private:

  double _threshold;
  int _ngroups;
  int _nresponse_elements;

};

#endif




