// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
//	test the EMOS redistribution retrieval
//
#include <cstdlib>
#include <valarray>
#include <iostream>
#include <cassert>
#include <string>
#include <stdio.h>
#include "CalServer.h"
#include "CalCommon.h"
#include "Redistribution.h"
#include "Xmm.h"
#include "CorePosData.h"
#include "ResponseCorrector.h"
#include "CalResponse.h"
#include "CalPNResponse.h"
#include "Rmfgen.h"
#include "PiledUpResponseCorrector.h"
#include "DummyPixelIterator.h"

using namespace std;

// From http://stackoverflow.com/questions/478898/how-to-execute-a-command-and-get-output-of-command-within-c
string exec(char* cmd) {
  FILE* pipe = popen(cmd, "r");
  if (!pipe) return "ERROR";
  char buffer[128];
  string result = "";
  while(!feof(pipe)) {
    if(fgets(buffer, 128, pipe) != NULL)
      result += buffer;
  }
  pclose(pipe);
  return result;
}

int main()
{
	//	setup the the Cal server
	const Xmm::Instrument instrument = Xmm::EPN;
	calServer.setInstrument(instrument);
	calServer.state()->ccd()->set(4);
        calServer.state()->accuracyLevel()->set(ELLBETA);

        // Create a simple response

        // Open a simple spectral file
        //string spectrumsetname="spec32.ds";

        string datadir=exec("saslocate lib/data/rmfgen_data");
        string spectrumsetname_bare="spectimerange.ds";
        string spectrumsetname=datadir+"/"+spectrumsetname_bare;
        DataSet * specSet = dataSetServer->open(spectrumsetname,
                                          DataSet::Read);

        Table * specTable = specSet->table("SPECTRUM");
        DatafileAttributes * specAtt = new DatafileAttributes(spectrumsetname);

        int specpix  = specTable->attribute("SPECPIX")->asInt();
        int specdelt  = specTable->attribute("SPECDELT")->asInt();
        double specval  = specTable->attribute("SPECVAL")->asReal();
        int detchans = specTable->attribute("DETCHANS")->asInt();

        double  offsetD = specval - (specpix*specdelt) - 0.5*(specdelt-1);
        unsigned offsetI   = int(offsetD);

        CorePosData * corePosData = new CorePosData(specSet,
                                           specTable,  specAtt);

        GenericResponse * resp = new CalPNResponse(corePosData->centre_raw(),
                                             offsetI,
                                             detchans,
                                             specdelt,
                                             corePosData->patternCode());

        // Create the response corrector 
        string eventfile_bare = "cleanevents04_1000.dat";
        string eventfile = datadir+"/"+eventfile_bare;

        PixelIterator * BSPixelIt = new DummyPixelIterator(-2100,2099,10.,
                                       -2100,2099,10.);

        ResponseCorrector  * r = new PiledUpResponseCorrector(resp,
                                  eventfile, corePosData, offsetI,
                                  specdelt, BSPixelIt);
        
       // Get response for 1 interval
        Interval<double> ebin0 = resp->ebins(318);

//        double energy0 = 0.5 * ( ebin0.lo() + ebin0.hi() );

       // Make the test events
        vector<double> corr = r->correctedResponse(ebin0,
                                       resp->responseVector(ebin0));
        vector<double> uncorr = resp->responseVector(ebin0);

       // Show the answer
        for (int i=80; i<=100; ++i)
          cout<<uncorr[i]<<" "<<corr[i]<<endl;

        cout<<endl;

	return 0;
}







